@@ -1,12 +1,51 @@
 use colored::*;
-use std::ops::Range;
+use std::{fmt::Write, ops::Range};
 
-/// Prints an error message along with the location in the source file where the error occurred
 pub fn pretty_print_error(
     message: &str,
     span: Range<usize>,
     source: &str,
     filename: std::path::PathBuf,
+    printer: &mut impl Write,
+) -> () {
+    pretty_print_diagnostic(
+        message,
+        span,
+        source,
+        filename,
+        printer,
+        "ERROR",
+        Color::Red,
+    );
+}
+
+pub fn pretty_print_warning(
+    message: &str,
+    span: Range<usize>,
+    source: &str,
+    filename: std::path::PathBuf,
+    printer: &mut impl Write,
+) -> () {
+    pretty_print_diagnostic(
+        message,
+        span,
+        source,
+        filename,
+        printer,
+        "WARNING",
+        Color::Yellow,
+    );
+}
+
+/// Prints an error message along with the location in the source file where the error occurred
+fn pretty_print_diagnostic(
+    message: &str,
+    span: Range<usize>,
+    source: &str,
+    filename: std::path::PathBuf,
+    printer: &mut impl Write,
+    diagnostic_type: &str,
+    color: Color,
 ) -> () {
     // Find the start and end of the error as line/col pair.
     let (start_line, start_col) = index_to_file_position(source, span.start);
@@ -15,12 +54,21 @@ pub fn pretty_print_error(
     let lines: Vec<_> = source.split("\n").collect();
 
     // let the user know there has been an error
-    println!("ERROR in {:?}: {}", filename, message);
+    let _ = writeln!(
+        printer,
+        "{} in {:?}: {}",
+        diagnostic_type.color(color),
+        filename,
+        message
+    );
+    if span == (0..0) {
+        return;
+    }
 
     // Print the error location
     for i in start_line..(end_line + 1) {
         // print the line from the source file
-        println!("{:4} {} {}", i + 1, "|".blue().bold(), lines[i]);
+        let _ = writeln!(printer, "{:4} {} {}", i + 1, "|".blue().bold(), lines[i]);
 
         // print the error underline (some amount of blank followed by the underline)
         let blank_size;
@@ -41,12 +89,13 @@ pub fn pretty_print_error(
         if underline_size <= 0 {
             underline_size = 1;
         }
-        println!(
+        let _ = writeln!(
+            printer,
             "{:4} {} {}{}",
             "",
             "|".blue().bold(),
             " ".repeat(blank_size),
-            "^".repeat(underline_size).red().bold()
+            "^".repeat(underline_size).color(color).bold()
         );
     }
 }
@@ -76,7 +125,7 @@ pub fn index_to_file_position(source: &str, index: usize) -> (usize, usize) {
 pub fn add_position_info_to_filename(
     source: &str,
     index: usize,
-    filename: std::path::PathBuf,
+    filename: &std::path::PathBuf,
 ) -> String {
     let (line, column) = index_to_file_position(source, index);
     format!("{}:{}:{}", filename.display(), line, column)
